# (C) Copyright 2008-2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""
Version information for this Apptools distribution.

This file is autogenerated by the Apptools setup.py script.
"""

#: The full version of the package, including a development suffix
#: for unreleased versions of the package.
version = "5.0.0"

#: The Git revision from which this release was made.
git_revision = "0e51542048c1aac7ec230c4b773336886bf40792"

#: Flag whether this is a final release
is_released = True
